#!/usr/bin/python
#  $Id: fetching_lib.py,v 1.9 2005/04/20 06:49:11 bharring Exp $
#
#  Copyright (C) 2005
#
#  Brian Harring
#  ferringb@gentoo.org
#  see Copying in mirror-dist tarball for GPL-2 rules.
#
import Queue
import threading, thread
import os, time
import portage.process as portage_exec
import signal

class file_to_fetch(object):
	def __init__(self, filename, uri, extra=None):
		self.filename = filename
		try:	uri.next
		except AttributeError:
			raise TypeError("uri is not an iterable",uri)
		self.uris = uri
		self.extra_data = extra
		self.uri_processed = 0

class fetcher_thread(threading.Thread):
	def __init__(self, job_queue, completed_queue, failed_queue, fetch_callable):
		"""all queues must be a Queue.queue derivative
		kill_switch is a threading.Event instance
		fetch_callable must accept a file_to_fetch object"""

		for name, inst in (("job_queue", job_queue), ("completed_queue", completed_queue),\
			 ("failed_queue", failed_queue)):
			if not isinstance(inst, Queue.Queue):
				raise TypeError(name, inst)

		if not isinstance(job_queue, job_tracker):
			raise TypeError("job_queue isn't a job_tracker instance!")
		self.job_queue = job_queue
		self.completed_queue = completed_queue
		self.failed_queue = failed_queue
		self.fetch = fetch_callable
		threading.Thread.__init__(self)

	def run(self):
		while True:
			try:	file = self.job_queue.consume()
			except Queue.Empty:	continue
			except ForcedSeppuku:	return
			if file == None:
				import sys
				sys.exit(1)
			ret=False
			try:	ret=self.fetch(file)
			except SystemExit:
				raise
			except ForcedSeppuku:
				return
			except Exception, e:
				self.failed_queue.put( (file, e) )
				continue
			if ret == True:
				self.completed_queue.put(file)
			else:
				self.failed_queue.put( (file, ret) )
			

class fetcher(object):
	def __init__(self, fetch_string, location, pid_manager=None, uid=None, gid=None, logging_dir=None):
		"""fetch_string is akin to printf, first %s is directory, second  is uri
		location is final_location to transfer to.
		and either chucks an exception, or returns true if the file is fine."""

		if not isinstance(fetch_string, str):
			raise TypeError("fetch_string isn't a string (dar!)", fetch_string)
		if not os.path.isdir(location):
			raise Exception("%s is not a dir/valid location!" % location)
		self.location = location

		if len(fetch_string.split("%s")) != 3:
			raise Exception,"Fetch_string must have 2, and only 2 '%s' in it", fetch_string

		self.fetch_string = fetch_string
		if uid != None:
			if not isinstance(uid, int): 	raise TypeError("uid must be an int",uid)
			self.uid = uid
		else:	self.uid = os.getuid()
		if gid != None:
			if not isinstance(gid, int): 	raise TypeError("uid must be an int",uid)
			self.gid = gid
		else:	self.gid = os.getgid()

		self.pid_manager = pid_manager
		self.log_dir = logging_dir

	def __call__(self, file):
		if not isinstance(file, file_to_fetch):
			raise TypeError(file,"not a file_to_fetch instance/derivative")

		file_name = os.path.join(self.location, file.filename)

		for uri in file.uris:
			file.uri_processed += 1
			fetchstring = self.fetch_string % (file_name, uri)
			fds = {}

			if self.log_dir:
				fds[1] = os.open(os.path.join(self.log_dir,file.filename), os.O_APPEND|os.O_WRONLY|os.O_CREAT, 0644)
				fds[2] = fds[1]
				os.write(fds[1], "\n"+fetchstring+"\n")

			pid=portage_exec.spawn(fetchstring, fd_pipes=fds, returnpid=True)[-1]

			self.pid_manager.notify(pid)
			ret = None
			try:		ret = self.pid_manager.wait(pid)
			finally:	
				if self.log_dir:
					os.write(fds[1], "\n%s\n\n" % str(ret))
					os.close(fds[1])

			# wget allows 1 for -c without needing any more to dl.
			# abstraction obviously breaks down here.
#			print "ret=",ret,file_name
			if ret != 0:
				size = None
				try:	size = os.lstat(file_name).st_size
				except OSError: pass
				if size != None and size > file.extra_data.get("size",0L):
					try:	
						os.unlink(file_name)
#						print "removed %s" % file_name
					except OSError:	
#						print "failed removing %s" % file_name
						pass
				# on to another round.
			else:
				return True

		try:
			os.unlink(file_name)
		except OSError:
			pass

		if not file.uri_processed:
			return "no fetchable uris"
		return "all uris failed"


class UnfoundMirror(Exception):
	def __init__(self, uri):	self.uri=uri
	def __str__(self):		return "no mirror was found for %s" % self.uri


class mirror_uri_processor(object):
	def __init__(self, mirrors, max_tries=None):
		self.mirrors = mirrors
		self.max_tries = max_tries

	def __call__(self, filename, uri):
		if isinstance(uri, str):
			uri=(uri,)
		uri_list = []
		use_mirrors = False
		for x in uri:
			if x.startswith("mirror://"):
				eidx=x.find("/",9)
				if eidx == -1 or not self.mirrors.has_key(x[9:eidx]):
					raise UnfoundMirror(x)
				path = x[eidx+1:]
				uri_list.append((path, x[9:eidx].lstrip("/")))
				use_mirrors = True
			else:	
				uri_list.append((x, None))
		if not use_mirrors:
			return iter(tuple(uri))
		return self._iter_uri(uri_list)

	def _iter_uri(self, uri_list):
		for uri, mirror in uri_list:
			if mirror == None:
				yield uri
			else:
				if self.max_tries and len(self.mirrors[mirror]) > self.max_tries:
					l = self.max_tries
				else:
					l = len(self.mirrors[mirror])

				for x in self.mirrors[mirror][:l]:
					yield x.rstrip("/") + "/" + uri.lstrip("/")


class checksum_verifier(object):
	__slots__=("_chf_callable")
	def __init__(self, chf_callable):
		"""chf_callable should be a func that takes a positional arg, filepath.
		it's return should be True or False."""
		self._chf_callable = chf_callable
	def verify(self, file, size, chksum):
		"""all args are self explanatory.  return true if the chksum matches"""
		# get the simple checks out of the way.

		try:	st = os.stat(file)
		except (OSError, IOError):
#			print "%s failed cause of stat", file
			return False
		try:
			if st.st_size != long(size):
#				print "%s failed cause of size," % file,st.st_size,long(size)
				return False
		except TypeError:	
			return False

		# now do the intensive check
		try:	
			chk = str(self._chf_callable(file)[0]).lstrip().rstrip()
			ret= str(chksum).lstrip().rstrip() == chk
			if not ret:
#				print "%s failed cause of md5, '%s' != '%s'" % (file, chk, chksum)
				pass
			return ret

		except SystemExit:	raise
		except Exception:	return False
		

class ForcedSeppuku(Exception):	pass

class job_tracker(Queue.Queue):
	def __init__(self, worker_delay, processing_switch, kill_switch, *args, **kwargs):
		self.processing = processing_switch
		self.seppuku = kill_switch
		self.waiting_count = 0
		self.waiting_lock = threading.Lock()
		self.worker_delay = worker_delay
		Queue.Queue.__init__(self, *args, **kwargs)

	def consume(self):
		self.waiting_lock.acquire()
		self.waiting_count += 1
		self.waiting_lock.release()
		self.processing.wait()
		self.waiting_lock.acquire()
		self.waiting_count -= 1
		self.waiting_lock.release()
		if self.seppuku.isSet():
			raise ForcedSeppuku
		f = self.get(1, self.worker_delay)
		if f != None:
			return f
		raise Queue.Empty


	def publish(self, item):
		self.put(item)
		self.processing.set()

	def no_work_left(self, workers, delay=0.05):
		"""atomic check to see if work is left, if no work, then if all workers are waiting
		returns true when all are delayed, and no work_left.  otherwise, false
		note it does not resume work when it determines no work is left"""
		if not self.empty():
			return False
		# can't trust it if it's positive.  delay to find out.
		self.processing.clear()
		while workers != self.waiting_count:
			time.sleep(delay)
		# by this point, all workers are paused.
		if not self.empty():
			self.processing.set()
			return False
		return True

	def kill_the_workers(self):
		self.seppuku.set()
		self.processing.set()

class CentralPidWatcher(threading.Thread):

	def __init__(self, kill_switch, poll_interval=0.5):
		self.poll_rate = poll_interval
		self.seppuku = kill_switch
		self.__pids = {}
		self.__values = {}
		self.__int_lock = threading.Lock()
		threading.Thread.__init__(self)


	def run(self):
		self_pid = os.getpid()
		force_sleep = True
		while not self.seppuku.isSet():
			if force_sleep:
				time.sleep(0.5)

			ps = self.__wait_pid(-1)
			if ps == None:
				force_sleep = True
				continue
			pid, status = ps

			# lock, and update pids.
			self.__int_lock.acquire()
			self.__values[pid] = status
			if pid in self.__pids:
				self.__pids[pid].release()
			else:
				self.__pids[pid] = threading.Lock()
				# is alloced released
			self.__int_lock.release()
			# so the waiter should be going.
		# if we reach here, seppuku is on.  cleanse.
#		print "aparently seppuku is in effect"
		self.__int_lock.acquire()

		final_e=None
		for pid, lock in self.__pids.items():
			try:	
				print "cpwid killing %i" % pid
				self.__kill_pid(pid)
				lock.release()
				
			except Exception, e: final_e = e

		self.__int_lock.release()
		if final_e != None:
			raise final_e
#		print "so... I'm dead."		


	def notify(self, pid):
#		print "registering",pid,
		self.__int_lock.acquire()
		if self.seppuku.isSet():
#			print "nope, killing it.  seppuku in effect"
			try:		self.__kill_pid(pid)
			finally:	self.__int_lock.release()
			raise ForcedSeppuku()

		if pid not in self.__pids:
			# already caught the pid, or just hit a very bad conflict.
			# either way, it's released from the get go.
			self.__pids[pid] = threading.Lock()
			self.__pids[pid].acquire()
#		print "registered"
		self.__int_lock.release()


	def wait(self, pid):
		if not self.isAlive():
			return self.__alt_wait(pid)

#		print "waiting on %i" % pid
		self.__pids[pid].acquire()
		self.__int_lock.acquire()
		del self.__pids[pid]
		if self.seppuku.isSet():
#			print "seppuku'ing %i" % pid
			self.__int_lock.release()
			raise ForcedSeppuku()
		v=self.__values[pid]
		del self.__values[pid]
		self.__int_lock.release()
		return v


	def __alt_wait(self, pid):
		ps=None
		while ps == None and not self.seppuku.isSet():
			ps = self.__wait_pid(pid)
			if not ps:
				time.sleep(self.poll_rate)
		if ps == None:
			# told to die (Why can't you just accept me daddy!?!)
			# kill the pid.
			self.__kill_pid(pid)
			raise ForcedSeppuku()
		return ps[1]


	def __wait_pid(self, pid=-1):
		try:	pid, status = os.waitpid(pid, os.WNOHANG)
		except OSError, e:
			# I hate you child errno 10 exception.
			# this means no thread active.  should, at least.
#			print "caught exception.  wth,",e
			force_sleep = True
			return None

		if pid == 0:
			return None

#		print "caught one",pid,status
		if (status & 0xff) == 0:	status = status >> 8 		#exit code
		else:				status = (status & 0xff) << 8 	#sig

		return pid, status


	def __kill_pid(self, pid):
		try:	
			print "killing %i" % pid
			os.kill(pid, signal.SIGTERM)
			if os.waitpid(pid, os.WNOHANG)[1] == 0:
				# still kicking.  feisty.
				os.kill(pid, signal.SIGKILL)
				os.waitpid(pid, 0)

		except OSError, oe:
			if oe.errno != 10:
				raise oe

