#!/bin/bash

# Author: Brian Harring <ferringb@gentoo.org>
# Script runs mirror-dist to cleanse/update distfiles

#uncomment to enable debugging runs.
#DEBUGGING='yes'

export PYTHONWARNINGS=d
#PORTDIR=/home/gmirror/rsync
PORTDIR=/usr/portage
BASE_PATH=$(readlink -f "${BASH_SOURCE[0]%/*}")
PORTAGE_BASE_PATH=$BASE_PATH/misc/portage
PYTHONPATH=$PORTAGE_BASE_PATH/pym${PYTHONPATH:+:}${PYTHONPATH}
PATH=$PORTAGE_BASE_PATH/bin${PATH:+:}${PATH}
PORTAGE_CONFIGROOT=$BASE_PATH/misc/config_root
FEATURES="strict -metadata-transfer -fixpackages"
BASE="/home/distfiles"
PORTAGE_DEPCACHEDIR=$BASE/depcache
export FEATURES PORTDIR PORTAGE_DEPCACHEDIR PYTHONPATH PORTAGE_CONFIGROOT

GEN_DEATH_LIST="${BASE}/scripts/gen-death-list.py"
START_HOUR=$(date '+%H')
NEW_REPORT=$(($(date '+%k') < 4))
DELETION_LISTS_DIR="${BASE}/deletion-list/"

MIRRORDIST="nice -n 5 ${BASE}/scripts/mirror-dist/mirror-dist.py"
LOG_PATH="${BASE}/logs"
DATA_PATH="${BASE}/mirror-dist-data/"
DIST_PATH="/mnt/distfiles/distfiles"
DIST_LOCAL_PATH=/mnt/distfiles/distfiles-local
DIST_LOCAL_DEATH_DAYS=30
TMP_PATH=/mnt/distfiles/tmp
PURGATORY_DIR="/mnt/distfiles/distfiles-purgatory"
PURGATORY_DB="${DATA_PATH}/purgatory.db"
PURGATORY_DEATH_WATCH_INTERVAL=$((7 * 24 * 60 * 60))
DEATH_WATCH_INTERVAL=$((3 * 24 * 60 * 60))
VERIFICATION_METHOD="size"
#MIRROR_OVERRIDES="${DATA_PATH}/mirror_overrides"
FETCH_LOGSDIR="/home/distfiles/last-individual-fetch-logs/"
WHITELIST_DIR="/mnt/distfiles/distfiles-whitelist"

OPTS_MIRRORDIST_LOG="--failure-log ${LOG_PATH}/failure.log \
	--success-log ${LOG_PATH}/success.log \
	--general-log ${LOG_PATH}/general.log"


[[ -e $MIRROR_OVERRIDES ]] || unset MIRROR_OVERRIDES

# people do stupid things.  make this check tight.
if [[ -n $FETCH_LOGSDIR ]]; then
	[[ -d $FETCH_LOGS ]] && rm ${FETCH_LOGS}/* &> /dev/null
fi

# people do stupid things.  make this check tight.
[[ -n $TMP_PATH ]] && rm ${TMP_PATH}/* &> /dev/null

if [[ -d $WHITELIST_DIR ]]; then
	[[ -e ${BASE}/whitelist ]] && rm -rf ${BASE}/whitelist
	for x in ${WHITELIST_DIR}/*; do
		[[ ! -f ${x} ]] && continue
		cat ${x} >> ${BASE}/whitelist
	done
	for x in ${DATA_PATH}/*.whitelist; do
		cat ${x} >> ${BASE}/whitelist
	done
	WHITELIST_FILE="${BASE}/whitelist"
fi

OPTS_MIRRORDIST="--cleanse-distfiles \
	--death-list ${DATA_PATH}/deathlist.db \
	--death-watch-seconds ${DEATH_WATCH_INTERVAL} \
	--distfiles-tracking ${DATA_PATH}/url.db \
	--append \
	${PORTDIR+--porttree ${PORTDIR}} \
	${TMP_PATH+--temp-location ${TMP_PATH}} \
	${DIST_PATH+--distfiles ${DIST_PATH}} \
	${DIST_LOCAL_PATH+--distfiles-local ${DIST_LOCAL_PATH}} \
	${PURGATORY_DIR+--purgatory-dir ${PURGATORY_DIR}} \
	${PURGATORY_DEATH_WATCH_INTERVAL+--purgatory-death-watch-seconds ${PURGATORY_DEATH_WATCH_INTERVAL}} \
	${MIRROR_OVERRIDES+--mirror-overrides ${MIRROR_OVERRIDES}} \
	--threads 30 \
	--verify-existing-${VERIFICATION_METHOD} \
	--disable-central-pid-manager \
	${FETCH_LOGSDIR+--individual-fetch-logs ${FETCH_LOGSDIR}} \
	${WHITELIST_FILE+--whitelist ${WHITELIST_FILE}} \
	${PURGATORY_DB+--purgatory-db ${PURGATORY_DB}} \
	${DEBUGGING+--debug} \
	--mirror-skip gentoo \
	--nomirror-exemptions gentoo"

echo "---------------------------------------------------" >> "${LOG_PATH}/times.log"
echo "START mirror-dist  $(date -u)" >> "${LOG_PATH}/times.log"
rm "${LOG_PATH}/failure.log" &> /dev/null
if [[ $NEW_REPORT == 1 ]] ; then
	# 0-4 utc
	if [[ ! -f ${LOG_PATH}/.report_timestamp || -n $(find "${LOG_PATH}/.report_timestamp" -ctime +0 2>/dev/null) ]] ; then
		rm -f "${LOG_PATH}"/{success.log,general.log,.report_timestamp}
		> "${LOG_PATH}/.report_timestamp"
	fi
fi

echo ${MIRRORDIST} ${OPTS_MIRRORDIST_LOG} ${OPTS_MIRRORDIST} >/tmp/mirrordist
${MIRRORDIST} ${OPTS_MIRRORDIST_LOG} ${OPTS_MIRRORDIST}
ret=$?
echo "END   mirror-dist  $(date -u)" >> "${LOG_PATH}/times.log"
if [[ $? != 0 ]]; then
	echo "non zero exit code from mirror-dist, bailing!" >> "${LOG_PATH}/times.log"
	echo "---------------------------------------------------" >> "${LOG_PATH}/times.log"
	exit $(($?))
fi

echo "START death-list   $(date -u)" >> "${LOG_PATH}/times.log"
rm ${DELETION_LISTS_DIR}/* &> /dev/null
${GEN_DEATH_LIST} "${DATA_PATH}/deathlist.db" ${DEATH_WATCH_INTERVAL} ${START_HOUR} ${DELETION_LISTS_DIR} ${DATA_PATH}/url.db
ret=$?
find "$DIST_LOCAL_PATH" -type f -ctime +$DIST_LOCAL_DEATH_DAYS -exec rm {} \;
echo "END   death-list   $(date -u)" >> "${LOG_PATH}/times.log"

if [[ $? != 0 ]]; then
	echo "non zero exit code from deathlist readable generation!" >> "${LOG_PATH}/times.log"
	echo "---------------------------------------------------" >> "${LOG_PATH}/times.log"
	exit $(($?))
fi
echo "---------------------------------------------------" >> "${LOG_PATH}/times.log"

${BASE}/scripts/gen-report-xml.py ${LOG_PATH}/failure.log ${LOG_PATH}/success.log ${DELETION_LISTS_DIR}/ '%Y-%m-%d' \
	${DIST_PATH} ${BASE}/reports/failure.xml
${BASE}/scripts/gen-report-whitelist-xml.py "$WHITELIST_DIR" "${BASE}/reports/whitelists.xml"
exit 0
